@@ -1,6 +1,6 @@
 var searchData=
 [
-  ['column',['column',['../classeltlyy_1_1position.html#a5834760b2b29ec22bea4b5e8ab3b74b6',1,'eltlyy::position::column()'],['../classkripkeyy_1_1position.html#ab0a2181ebac3b2d484ed08cef3e4df68',1,'kripkeyy::position::column()'],['../classltlyy_1_1position.html#a4e34ca78e4f8d15cef154cb98f1d28dd',1,'ltlyy::position::column()'],['../classneverclaimyy_1_1position.html#aad933b0a66ce9e68c548be8e66a4471c',1,'neverclaimyy::position::column()'],['../classtgbayy_1_1position.html#a85d4b89078725a4c3073f88defbc98d6',1,'tgbayy::position::column()']]],
+  ['column',['column',['../classkripkeyy_1_1position.html#ab0a2181ebac3b2d484ed08cef3e4df68',1,'kripkeyy::position::column()'],['../classltlyy_1_1position.html#a4e34ca78e4f8d15cef154cb98f1d28dd',1,'ltlyy::position::column()'],['../classneverclaimyy_1_1position.html#aad933b0a66ce9e68c548be8e66a4471c',1,'neverclaimyy::position::column()'],['../classtgbayy_1_1position.html#a85d4b89078725a4c3073f88defbc98d6',1,'tgbayy::position::column()']]],
   ['condition',['condition',['../structspot_1_1scc__stack__ta_1_1connected__component.html#a7d5d3e5f5562d146745d036dcc795072',1,'spot::scc_stack_ta::connected_component::condition()'],['../structspot_1_1scc__stack_1_1connected__component.html#aa753864e5e55807ac67794a9999873b9',1,'spot::scc_stack::connected_component::condition()']]],
   ['conds',['conds',['../structspot_1_1scc__map_1_1scc.html#a81f43b7400f58d689f02374c9a91b9d9',1,'spot::scc_map::scc']]],
   ['count_5f',['count_',['../classspot_1_1ltl_1_1formula.html#a79d5a955150b1aa40ab396a5333eed7a',1,'spot::ltl::formula']]]
