@@ -1,6 +1,6 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2012, 2013 Laboratoire de Recherche et Développement
-// de l'Epita (LRDE).
+// Copyright (C) 2012, 2013, 2014 Laboratoire de Recherche et
+// Développement de l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
 //
@@ -94,6 +94,8 @@ Exit status:\n\
 #define OPT_COLOR 10
 #define OPT_NOCOMP 11
 #define OPT_OMIT 12
+#define OPT_BOGUS 13
+#define OPT_VERBOSE 14
 
 static const argp_option options[] =
   {
@@ -159,6 +161,10 @@ static const argp_option options[] =
       "colorize output; WHEN can be 'never', 'always' (the default if "
       "--color is used without argument), or "
       "'auto' (the default if --color is not used)", 0 },
+    { "save-bogus", OPT_BOGUS, "FILENAME", 0,
+      "save formulas for which problems were detected in FILENAME", 0 },
+    { "verbose", OPT_VERBOSE, 0, 0,
+      "print what is being done, for debugging", 0 },
     { 0, 0, 0, 0, 0, 0 }
   };
 
@@ -187,7 +193,7 @@ ARGMATCH_VERIFY(color_args, color_types);
 
 color_type color_opt = color_if_tty;
 const char* bright_red = "\033[01;31m";
-const char* bright_white = "\033[01;37m";
+const char* bright_blue = "\033[01;34m";
 const char* bright_yellow = "\033[01;33m";
 const char* reset_color = "\033[m";
 
@@ -206,6 +212,9 @@ unsigned products = 1;
 bool products_avg = true;
 bool opt_omit = false;
 bool has_sr = false; // Has Streett or Rabin automata to process.
+const char* bogus_output_filename = 0;
+std::ofstream* bogus_output = 0;
+bool verbose = false;
 
 struct translator_spec
 {
@@ -509,6 +518,14 @@ parse_opt(int key, char* arg, struct argp_state*)
 		<< "on your platform" << std::endl;
 #endif
       break;
+    case OPT_BOGUS:
+      {
+	bogus_output = new std::ofstream(arg);
+	if (!*bogus_output)
+	  error(2, errno, "cannot open '%s'", arg);
+	bogus_output_filename = arg;
+	break;
+      }
     case OPT_COLOR:
       {
 	if (arg)
@@ -558,6 +575,9 @@ parse_opt(int key, char* arg, struct argp_state*)
     case OPT_STOP_ERR:
       stop_on_error = true;
       break;
+    case OPT_VERBOSE:
+      verbose = true;
+      break;
     default:
       return ARGP_ERR_UNKNOWN;
     }
@@ -849,7 +869,8 @@ namespace
     }
 
     const spot::tgba*
-    translate(unsigned int translator_num, char l, statistics_formula* fstats)
+    translate(unsigned int translator_num, char l, statistics_formula* fstats,
+	      bool& problem)
     {
       output.reset(translator_num);
 
@@ -874,11 +895,13 @@ namespace
 	  std::cerr << "warning: timeout during execution of command\n";
 	  ++timeout_count;
 	  status_str = "timeout";
+	  problem = false;	// A timeout is not a sign of a bug
 	  es = -1;
 	}
       else if (WIFSIGNALED(es))
 	{
 	  status_str = "signal";
+	  problem = true;
 	  es = WTERMSIG(es);
 	  global_error() << "error: execution terminated by signal "
 			 << es << ".\n";
@@ -888,6 +911,7 @@ namespace
 	{
 	  es = WEXITSTATUS(es);
 	  status_str = "exit code";
+	  problem = true;
 	  global_error() << "error: execution returned exit code "
 			 << es << ".\n";
 	  end_error();
@@ -895,6 +919,7 @@ namespace
       else
 	{
 	  status_str = "ok";
+	  problem = false;
 	  es = 0;
 	  switch (output.format)
 	    {
@@ -906,6 +931,7 @@ namespace
 		if (!pel.empty())
 		  {
 		    status_str = "parse error";
+		    problem = true;
 		    es = -1;
 		    std::ostream& err = global_error();
 		    err << "error: failed to parse the produced neverclaim.\n";
@@ -923,6 +949,7 @@ namespace
 		if (!f)
 		  {
 		    status_str = "no output";
+		    problem = true;
 		    es = -1;
 		    global_error() << "Cannot open " << output.val()
 				   << std::endl;
@@ -934,6 +961,7 @@ namespace
 		    if (!res)
 		      {
 			status_str = "parse error";
+			problem = true;
 			es = -1;
 			global_error() << ("error: failed to parse output in "
 					   "LBTT format: ")
@@ -952,6 +980,7 @@ namespace
 		if (!pel.empty())
 		  {
 		    status_str = "parse error";
+		    problem = true;
 		    es = -1;
 		    std::ostream& err = global_error();
 		    err << "error: failed to parse the produced DSTAR"
@@ -963,22 +992,24 @@ namespace
 		  }
 		else
 		  {
+		    const char* type = 0;
+		    switch (aut->type)
+		      {
+		      case spot::Rabin:
+			type = "DRA";
+			break;
+		      case spot::Streett:
+			type = "DSA";
+			break;
+		      }
+		    assert(type);
+
 		    // Gather statistics about the input automaton
 		    if (want_stats)
 		      {
 			statistics* st = &(*fstats)[translator_num];
 			st->has_in = true;
-
-			switch (aut->type)
-			  {
-			  case spot::Rabin:
-			    st->in_type = "DRA";
-			    break;
-			  case spot::Streett:
-			    st->in_type = "DSA";
-			    break;
-			  }
-
+			st->in_type = type;
 			spot::tgba_sub_statistics s =
 			  sub_stats_reachable(aut->aut);
 			st->in_states= s.states;
@@ -991,6 +1022,8 @@ namespace
 			st->in_scc = m.scc_count();
 		      }
 		    // convert it into TGBA for further processing
+		    if (verbose)
+		      std::cerr << "info: converting " << type << " to TGBA\n";
 		    res = dstar_to_tgba(aut);
 		    delete aut;
 		  }
@@ -1012,6 +1045,8 @@ namespace
 	  // Compute statistics.
 	  if (res)
 	    {
+	      if (verbose)
+		std::cerr << "info: getting statistics\n";
 	      st->ok = true;
 	      spot::tgba_sub_statistics s = sub_stats_reachable(res);
 	      st->states = s.states;
@@ -1048,7 +1083,7 @@ namespace
     }
   };
 
-  static void
+  static bool
   check_empty_prod(const spot::tgba* aut_i, const spot::tgba* aut_j,
 		   size_t i, size_t j, bool icomp, bool jcomp)
   {
@@ -1056,6 +1091,20 @@ namespace
     spot::emptiness_check* ec = spot::couvreur99(prod);
     spot::emptiness_check_result* res = ec->check();
 
+    if (verbose)
+      {
+	std::cerr << "info: check_empty ";
+	if (icomp)
+	  std::cerr << "Comp(N" << i << ')';
+	else
+	  std::cerr << "P" << i;
+	if (jcomp)
+	  std::cerr << "*Comp(P" << j << ')';
+	else
+	  std::cerr << "*N" << j;
+	std::cerr << '\n';
+      }
+
     if (res)
       {
 	std::ostream& err = global_error();
@@ -1091,12 +1140,27 @@ namespace
     delete res;
     delete ec;
     delete prod;
+    return res;
   }
 
-  static void
+  static bool
   cross_check(const std::vector<spot::scc_map*>& maps, char l, unsigned p)
   {
     size_t m = maps.size();
+    if (verbose)
+      {
+	std::cerr << "info: cross_check {";
+	bool first = true;
+	for (size_t i = 0; i < m; ++i)
+	  {
+	    if (first)
+	      first = false;
+	    else
+	      std::cerr << ',';
+	    std::cerr << l << i;
+	  }
+	std::cerr << "}, state-space #" << p << '/' << products << "\n";
+      }
 
     std::vector<bool> res(m);
     unsigned verified = 0;
@@ -1146,7 +1210,9 @@ namespace
 	else
 	  err << "the state-space\n";
 	end_error();
+	return true;
       }
+    return false;
   }
 
   typedef std::set<spot::state*, spot::state_ptr_less_than> state_set;
@@ -1212,12 +1278,35 @@ namespace
 	(*i++)->destroy();
     }
 
+    int
+    process_string(const std::string& input,
+		   const char* filename,
+		   int linenum)
+    {
+      spot::ltl::parse_error_list pel;
+      const spot::ltl::formula* f = parse_formula(input, pel);
+
+      if (!f || !pel.empty())
+	{
+	  if (filename)
+	    error_at_line(0, 0, filename, linenum, "parse error:");
+	  spot::ltl::format_parse_errors(std::cerr, input, pel);
+	  if (f)
+	    f->destroy();
+	  return 1;
+	}
+      int res = process_formula(f, filename, linenum);
+
+      if (res && bogus_output)
+	*bogus_output << input << std::endl;
+      return 0;
+    }
+
+
     int
     process_formula(const spot::ltl::formula* f,
 		    const char* filename = 0, int linenum = 0)
     {
-      (void) filename;
-      (void) linenum;
       static unsigned round = 0;
 
       // If we need LBT atomic proposition in any of the input or
@@ -1244,7 +1333,7 @@ namespace
       if (filename || linenum)
 	std::cerr << " ";
       if (color_opt)
-	std::cerr << bright_white;
+	std::cerr << bright_blue;
       std::cerr << fstr << "\n";
       if (color_opt)
 	std::cerr << reset_color;
@@ -1268,6 +1357,9 @@ namespace
 	    }
 	}
 
+
+      int problems = 0;
+
       // These store the result of the translation of the positive and
       // negative formulas.
       size_t m = translators.size();
@@ -1288,7 +1380,10 @@ namespace
 
       for (size_t n = 0; n < m; ++n)
 	{
-	  pos[n] = runner.translate(n, 'P', pstats);
+	  bool prob;
+	  pos[n] = runner.translate(n, 'P', pstats, prob);
+	  problems += prob;
+
 	  // If the automaton is deterministic, compute its complement
 	  // as well.  Note that if we have computed statistics
 	  // already, there is no need to call is_deterministic()
@@ -1326,7 +1421,10 @@ namespace
 
 	  for (size_t n = 0; n < m; ++n)
 	    {
-	      neg[n] = runner.translate(n, 'N', nstats);
+	      bool prob;
+	      neg[n] = runner.translate(n, 'N', nstats, prob);
+	      problems += prob;
+
 	      // If the automaton is deterministic, compute its
 	      // complement as well.  Note that if we have computed
 	      // statistics already, there is no need to call
@@ -1353,7 +1451,8 @@ namespace
 	      for (size_t j = 0; j < m; ++j)
 		if (neg[j])
 		  {
-		    check_empty_prod(pos[i], neg[j], i, j, false, false);
+		    problems +=
+		      check_empty_prod(pos[i], neg[j], i, j, false, false);
 
 		    // Deal with the extra complemented automata if we
 		    // have some.
@@ -1371,13 +1470,18 @@ namespace
 		    // translation was not deterministic.
 
 		    if (i != j && comp_pos[j] && !comp_neg[j])
-		      check_empty_prod(pos[i], comp_pos[j], i, j, false, true);
-		    if (i != j && comp_neg[i] && !comp_neg[i])
-		      check_empty_prod(comp_neg[i], neg[j], i, j, true, false);
+		      problems +=
+			check_empty_prod(pos[i], comp_pos[j],
+					 i, j, false, true);
+		    if (i != j && comp_neg[i] && !comp_pos[i])
+		      problems +=
+			check_empty_prod(comp_neg[i], neg[j],
+					 i, j, true, false);
 		    if (comp_pos[i] && comp_neg[j] &&
 			(i == j || (!comp_neg[i] && !comp_pos[j])))
-		      check_empty_prod(comp_pos[i], comp_neg[j],
-				       i, j, true, true);
+		      problems +=
+			check_empty_prod(comp_pos[i], comp_neg[j],
+					 i, j, true, true);
 		  }
 	}
       else
@@ -1405,6 +1509,10 @@ namespace
 	{
 	  // build a random state-space.
 	  spot::srand(seed);
+	  if (verbose)
+	    std::cerr << "info: building state-space #" << p << '/' << products
+		      << " with seed " << seed << '\n';
+
 	  spot::tgba* statespace = spot::random_graph(states, density,
 						      ap, &dict);
 
@@ -1457,23 +1565,32 @@ namespace
 	  if (!no_checks)
 	    {
 	      // cross-comparison test
-	      cross_check(pos_map, 'P', p);
-	      cross_check(neg_map, 'N', p);
+	      problems += cross_check(pos_map, 'P', p);
+	      problems += cross_check(neg_map, 'N', p);
 
 	      // consistency check
 	      for (size_t i = 0; i < m; ++i)
-		if (pos_map[i] && neg_map[i] &&
-		    !(consistency_check(pos_map[i], neg_map[i], statespace)))
+		if (pos_map[i] && neg_map[i])
 		  {
-		    std::ostream& err = global_error();
-		    err << "error: inconsistency between P" << i
-			<< " and N" << i;
-		    if (products > 1)
-		      err << " for state-space #" << p
-			  << "/" << products << "\n";
-		    else
-		      err << "\n";
-		    end_error();
+		    if (verbose)
+		      std::cerr << "info: consistency_check (P" << i
+				<< ",N" << i << "), state-space #"
+				<< p << '/' << products << '\n';
+		    if (!(consistency_check(pos_map[i], neg_map[i],
+					    statespace)))
+		      {
+			++problems;
+
+			std::ostream& err = global_error();
+			err << "error: inconsistency between P" << i
+			    << " and N" << i;
+			if (products > 1)
+			  err << " for state-space #" << p
+			      << "/" << products << "\n";
+			else
+			  err << "\n";
+			end_error();
+		      }
 		  }
 	    }
 
@@ -1507,7 +1624,7 @@ namespace
 
       // Shall we stop processing formulas now?
       abort_run = global_error_flag && stop_on_error;
-      return 0;
+      return problems;
     }
   };
 }
@@ -1516,6 +1633,9 @@ namespace
 static void
 print_stats_csv(const char* filename)
 {
+  if (verbose)
+    std::cerr << "info: writing CSV to " << filename << '\n';
+
   std::ofstream* outfile = 0;
   std::ostream* out;
   if (!strncmp(filename, "-", 2))
@@ -1525,7 +1645,7 @@ print_stats_csv(const char* filename)
   else
     {
       out = outfile = new std::ofstream(filename);
-      if (!outfile)
+      if (!*outfile)
 	error(2, errno, "cannot open '%s'", filename);
     }
 
@@ -1554,6 +1674,9 @@ print_stats_csv(const char* filename)
 static void
 print_stats_json(const char* filename)
 {
+  if (verbose)
+    std::cerr << "info: writing JSON to " << filename << '\n';
+
   std::ofstream* outfile = 0;
   std::ostream* out;
   if (!strncmp(filename, "-", 2))
@@ -1563,7 +1686,7 @@ print_stats_json(const char* filename)
   else
     {
       out = outfile = new std::ofstream(filename);
-      if (!outfile)
+      if (!*outfile)
 	error(2, errno, "cannot open '%s'", filename);
     }
 
@@ -1642,10 +1765,16 @@ main(int argc, char** argv)
       if (global_error_flag)
 	{
 	  std::ostream& err = global_error();
-	  err << ("error: some error was detected during the above runs,\n"
-		  "       please search for 'error:' messages in the above "
-		  "trace.")
-	      << std::endl;
+	  if (bogus_output)
+	    err << ("error: some error was detected during the above runs.\n"
+		    "       Check file ")
+		<< bogus_output_filename
+		<< " for problematic formulas.";
+	  else
+	    err << ("error: some error was detected during the above runs,\n"
+		    "       please search for 'error:' messages in the above"
+		    " trace.");
+	  err << std::endl;
 	  if (timeout_count == 1)
 	    err << "Additionally, 1 timeout occurred." << std::endl;
 	  else if (timeout_count > 1)
@@ -1662,6 +1791,8 @@ main(int argc, char** argv)
 		  << " timeouts, but no other problem detected." << std::endl;
     }
 
+  delete bogus_output;
+
   if (json_output)
     print_stats_json(json_output);
   if (csv_output)
