@@ -63,8 +63,9 @@
 using namespace spot::ltl;
 static bool accept_all_needed = false;
 static bool accept_all_seen = false;
+static std::map<std::string, spot::location> labels;
 
-#line 68 "neverclaimparse.cc" // lalr1.cc:408
+#line 69 "neverclaimparse.cc" // lalr1.cc:408
 
 
 #ifndef YY_
@@ -150,7 +151,7 @@ static bool accept_all_seen = false;
 
 
 namespace neverclaimyy {
-#line 154 "neverclaimparse.cc" // lalr1.cc:474
+#line 155 "neverclaimparse.cc" // lalr1.cc:474
 
   /* Return YYSTR after stripping away unnecessary quotes and
      backslashes, so that it's suitable for yyerror.  The heuristic is
@@ -369,28 +370,35 @@ namespace neverclaimyy {
     {
             case 14: // "boolean formula"
 
-#line 83 "neverclaimparse.yy" // lalr1.cc:599
+#line 84 "neverclaimparse.yy" // lalr1.cc:599
         { delete (yysym.value.str); }
-#line 375 "neverclaimparse.cc" // lalr1.cc:599
+#line 376 "neverclaimparse.cc" // lalr1.cc:599
         break;
 
       case 15: // "identifier"
 
-#line 83 "neverclaimparse.yy" // lalr1.cc:599
+#line 84 "neverclaimparse.yy" // lalr1.cc:599
         { delete (yysym.value.str); }
-#line 382 "neverclaimparse.cc" // lalr1.cc:599
+#line 383 "neverclaimparse.cc" // lalr1.cc:599
         break;
 
-      case 23: // ident_list
+      case 23: // one_ident
 
-#line 83 "neverclaimparse.yy" // lalr1.cc:599
+#line 84 "neverclaimparse.yy" // lalr1.cc:599
         { delete (yysym.value.str); }
-#line 389 "neverclaimparse.cc" // lalr1.cc:599
+#line 390 "neverclaimparse.cc" // lalr1.cc:599
         break;
 
-      case 24: // transition_block
+      case 24: // ident_list
 
-#line 85 "neverclaimparse.yy" // lalr1.cc:599
+#line 84 "neverclaimparse.yy" // lalr1.cc:599
+        { delete (yysym.value.str); }
+#line 397 "neverclaimparse.cc" // lalr1.cc:599
+        break;
+
+      case 25: // transition_block
+
+#line 86 "neverclaimparse.yy" // lalr1.cc:599
         {
   for (std::list<pair>::iterator i = (yysym.value.list)->begin();
        i != (yysym.value.list)->end(); ++i)
@@ -400,12 +408,12 @@ namespace neverclaimyy {
   }
   delete (yysym.value.list);
   }
-#line 404 "neverclaimparse.cc" // lalr1.cc:599
+#line 412 "neverclaimparse.cc" // lalr1.cc:599
         break;
 
-      case 26: // transitions
+      case 27: // transitions
 
-#line 85 "neverclaimparse.yy" // lalr1.cc:599
+#line 86 "neverclaimparse.yy" // lalr1.cc:599
         {
   for (std::list<pair>::iterator i = (yysym.value.list)->begin();
        i != (yysym.value.list)->end(); ++i)
@@ -415,35 +423,35 @@ namespace neverclaimyy {
   }
   delete (yysym.value.list);
   }
-#line 419 "neverclaimparse.cc" // lalr1.cc:599
+#line 427 "neverclaimparse.cc" // lalr1.cc:599
         break;
 
-      case 27: // formula
-
-#line 83 "neverclaimparse.yy" // lalr1.cc:599
-        { delete (yysym.value.str); }
-#line 426 "neverclaimparse.cc" // lalr1.cc:599
-        break;
-
-      case 28: // opt_dest
-
-#line 83 "neverclaimparse.yy" // lalr1.cc:599
-        { delete (yysym.value.str); }
-#line 433 "neverclaimparse.cc" // lalr1.cc:599
-        break;
-
-      case 29: // src_dest
+      case 28: // formula
 
 #line 84 "neverclaimparse.yy" // lalr1.cc:599
-        { (yysym.value.p)->first->destroy(); delete (yysym.value.p)->second; delete (yysym.value.p); }
-#line 440 "neverclaimparse.cc" // lalr1.cc:599
+        { delete (yysym.value.str); }
+#line 434 "neverclaimparse.cc" // lalr1.cc:599
         break;
 
-      case 30: // transition
+      case 29: // opt_dest
 
 #line 84 "neverclaimparse.yy" // lalr1.cc:599
+        { delete (yysym.value.str); }
+#line 441 "neverclaimparse.cc" // lalr1.cc:599
+        break;
+
+      case 30: // src_dest
+
+#line 85 "neverclaimparse.yy" // lalr1.cc:599
         { (yysym.value.p)->first->destroy(); delete (yysym.value.p)->second; delete (yysym.value.p); }
-#line 447 "neverclaimparse.cc" // lalr1.cc:599
+#line 448 "neverclaimparse.cc" // lalr1.cc:599
+        break;
+
+      case 31: // transition
+
+#line 85 "neverclaimparse.yy" // lalr1.cc:599
+        { (yysym.value.p)->first->destroy(); delete (yysym.value.p)->second; delete (yysym.value.p); }
+#line 455 "neverclaimparse.cc" // lalr1.cc:599
         break;
 
 
@@ -468,57 +476,68 @@ namespace neverclaimyy {
     {
             case 14: // "boolean formula"
 
-#line 94 "neverclaimparse.yy" // lalr1.cc:617
+#line 95 "neverclaimparse.yy" // lalr1.cc:617
         {
     if ((yysym.value.str))
       debug_stream() << *(yysym.value.str);
     else
       debug_stream() << "\"\""; }
-#line 478 "neverclaimparse.cc" // lalr1.cc:617
+#line 486 "neverclaimparse.cc" // lalr1.cc:617
         break;
 
       case 15: // "identifier"
 
-#line 94 "neverclaimparse.yy" // lalr1.cc:617
+#line 95 "neverclaimparse.yy" // lalr1.cc:617
         {
     if ((yysym.value.str))
       debug_stream() << *(yysym.value.str);
     else
       debug_stream() << "\"\""; }
-#line 489 "neverclaimparse.cc" // lalr1.cc:617
+#line 497 "neverclaimparse.cc" // lalr1.cc:617
         break;
 
-      case 23: // ident_list
+      case 23: // one_ident
 
-#line 94 "neverclaimparse.yy" // lalr1.cc:617
+#line 95 "neverclaimparse.yy" // lalr1.cc:617
         {
     if ((yysym.value.str))
       debug_stream() << *(yysym.value.str);
     else
       debug_stream() << "\"\""; }
-#line 500 "neverclaimparse.cc" // lalr1.cc:617
+#line 508 "neverclaimparse.cc" // lalr1.cc:617
         break;
 
-      case 27: // formula
+      case 24: // ident_list
 
-#line 94 "neverclaimparse.yy" // lalr1.cc:617
+#line 95 "neverclaimparse.yy" // lalr1.cc:617
         {
     if ((yysym.value.str))
       debug_stream() << *(yysym.value.str);
     else
       debug_stream() << "\"\""; }
-#line 511 "neverclaimparse.cc" // lalr1.cc:617
+#line 519 "neverclaimparse.cc" // lalr1.cc:617
         break;
 
-      case 28: // opt_dest
+      case 28: // formula
 
-#line 94 "neverclaimparse.yy" // lalr1.cc:617
+#line 95 "neverclaimparse.yy" // lalr1.cc:617
         {
     if ((yysym.value.str))
       debug_stream() << *(yysym.value.str);
     else
       debug_stream() << "\"\""; }
-#line 522 "neverclaimparse.cc" // lalr1.cc:617
+#line 530 "neverclaimparse.cc" // lalr1.cc:617
+        break;
+
+      case 29: // opt_dest
+
+#line 95 "neverclaimparse.yy" // lalr1.cc:617
+        {
+    if ((yysym.value.str))
+      debug_stream() << *(yysym.value.str);
+    else
+      debug_stream() << "\"\""; }
+#line 541 "neverclaimparse.cc" // lalr1.cc:617
         break;
 
 
@@ -745,48 +764,55 @@ namespace neverclaimyy {
   case 7:
 #line 113 "neverclaimparse.yy" // lalr1.cc:847
     {
+      std::pair<std::map<std::string, spot::location>::const_iterator, bool>
+	res = labels.insert(std::make_pair(*(yystack_[1].value.str), yystack_[1].location));
+      if (!res.second)
+	{
+	  error(yystack_[1].location, std::string("redefinition of ") + *(yystack_[1].value.str) + "...");
+	  error(res.first->second, std::string("... ")  + *(yystack_[1].value.str) + " previously defined here");
+	}
       (yylhs.value.str) = (yystack_[1].value.str);
     }
-#line 751 "neverclaimparse.cc" // lalr1.cc:847
-    break;
-
-  case 8:
-#line 117 "neverclaimparse.yy" // lalr1.cc:847
-    {
-      result->add_state_alias(*(yystack_[1].value.str), *(yystack_[2].value.str));
-      // Keep any identifier that starts with accept.
-      if (strncmp("accept", (yystack_[2].value.str)->c_str(), 6))
-        {
-          delete (yystack_[2].value.str);
-          (yylhs.value.str) = (yystack_[1].value.str);
-        }
-      else
-        {
-	  delete (yystack_[1].value.str);
-	  (yylhs.value.str) = (yystack_[2].value.str);
-        }
-    }
-#line 770 "neverclaimparse.cc" // lalr1.cc:847
+#line 777 "neverclaimparse.cc" // lalr1.cc:847
     break;
 
   case 9:
-#line 135 "neverclaimparse.yy" // lalr1.cc:847
+#line 127 "neverclaimparse.yy" // lalr1.cc:847
     {
-      (yylhs.value.list) = (yystack_[1].value.list);
+      result->add_state_alias(*(yystack_[0].value.str), *(yystack_[1].value.str));
+      // Keep any identifier that starts with accept.
+      if (strncmp("accept", (yystack_[1].value.str)->c_str(), 6))
+        {
+          delete (yystack_[1].value.str);
+          (yylhs.value.str) = (yystack_[0].value.str);
+        }
+      else
+        {
+	  delete (yystack_[0].value.str);
+	  (yylhs.value.str) = (yystack_[1].value.str);
+        }
     }
-#line 778 "neverclaimparse.cc" // lalr1.cc:847
+#line 796 "neverclaimparse.cc" // lalr1.cc:847
     break;
 
   case 10:
-#line 139 "neverclaimparse.yy" // lalr1.cc:847
+#line 145 "neverclaimparse.yy" // lalr1.cc:847
     {
       (yylhs.value.list) = (yystack_[1].value.list);
     }
-#line 786 "neverclaimparse.cc" // lalr1.cc:847
+#line 804 "neverclaimparse.cc" // lalr1.cc:847
     break;
 
   case 11:
-#line 145 "neverclaimparse.yy" // lalr1.cc:847
+#line 149 "neverclaimparse.yy" // lalr1.cc:847
+    {
+      (yylhs.value.list) = (yystack_[1].value.list);
+    }
+#line 812 "neverclaimparse.cc" // lalr1.cc:847
+    break;
+
+  case 12:
+#line 155 "neverclaimparse.yy" // lalr1.cc:847
     {
       if (*(yystack_[1].value.str) == "accept_all")
 	accept_all_seen = true;
@@ -798,23 +824,23 @@ namespace neverclaimyy {
 					 spot::ltl::constant::true_instance());
       delete (yystack_[1].value.str);
     }
-#line 802 "neverclaimparse.cc" // lalr1.cc:847
-    break;
-
-  case 12:
-#line 156 "neverclaimparse.yy" // lalr1.cc:847
-    { delete (yystack_[0].value.str); }
-#line 808 "neverclaimparse.cc" // lalr1.cc:847
+#line 828 "neverclaimparse.cc" // lalr1.cc:847
     break;
 
   case 13:
-#line 157 "neverclaimparse.yy" // lalr1.cc:847
-    { delete (yystack_[1].value.str); }
-#line 814 "neverclaimparse.cc" // lalr1.cc:847
+#line 166 "neverclaimparse.yy" // lalr1.cc:847
+    { delete (yystack_[0].value.str); }
+#line 834 "neverclaimparse.cc" // lalr1.cc:847
     break;
 
   case 14:
-#line 159 "neverclaimparse.yy" // lalr1.cc:847
+#line 167 "neverclaimparse.yy" // lalr1.cc:847
+    { delete (yystack_[1].value.str); }
+#line 840 "neverclaimparse.cc" // lalr1.cc:847
+    break;
+
+  case 15:
+#line 169 "neverclaimparse.yy" // lalr1.cc:847
     {
       std::list<pair>::iterator it;
       bool acc = !strncmp("accept", (yystack_[1].value.str)->c_str(), 6);
@@ -835,17 +861,17 @@ namespace neverclaimyy {
 	delete it->second;
       delete (yystack_[0].value.list);
     }
-#line 839 "neverclaimparse.cc" // lalr1.cc:847
-    break;
-
-  case 15:
-#line 182 "neverclaimparse.yy" // lalr1.cc:847
-    { (yylhs.value.list) = new std::list<pair>; }
-#line 845 "neverclaimparse.cc" // lalr1.cc:847
+#line 865 "neverclaimparse.cc" // lalr1.cc:847
     break;
 
   case 16:
-#line 184 "neverclaimparse.yy" // lalr1.cc:847
+#line 192 "neverclaimparse.yy" // lalr1.cc:847
+    { (yylhs.value.list) = new std::list<pair>; }
+#line 871 "neverclaimparse.cc" // lalr1.cc:847
+    break;
+
+  case 17:
+#line 194 "neverclaimparse.yy" // lalr1.cc:847
     {
       if ((yystack_[0].value.p))
 	{
@@ -854,43 +880,43 @@ namespace neverclaimyy {
 	}
       (yylhs.value.list) = (yystack_[1].value.list);
     }
-#line 858 "neverclaimparse.cc" // lalr1.cc:847
-    break;
-
-  case 18:
-#line 194 "neverclaimparse.yy" // lalr1.cc:847
-    { (yylhs.value.str) = new std::string("0"); }
-#line 864 "neverclaimparse.cc" // lalr1.cc:847
-    break;
-
-  case 19:
-#line 198 "neverclaimparse.yy" // lalr1.cc:847
-    {
-      (yylhs.value.str) = 0;
-    }
-#line 872 "neverclaimparse.cc" // lalr1.cc:847
+#line 884 "neverclaimparse.cc" // lalr1.cc:847
     break;
 
   case 20:
-#line 202 "neverclaimparse.yy" // lalr1.cc:847
-    {
-      (yylhs.value.str) = (yystack_[0].value.str);
-    }
-#line 880 "neverclaimparse.cc" // lalr1.cc:847
+#line 204 "neverclaimparse.yy" // lalr1.cc:847
+    { (yylhs.value.str) = new std::string("0"); }
+#line 890 "neverclaimparse.cc" // lalr1.cc:847
     break;
 
   case 21:
-#line 206 "neverclaimparse.yy" // lalr1.cc:847
+#line 208 "neverclaimparse.yy" // lalr1.cc:847
+    {
+      (yylhs.value.str) = 0;
+    }
+#line 898 "neverclaimparse.cc" // lalr1.cc:847
+    break;
+
+  case 22:
+#line 212 "neverclaimparse.yy" // lalr1.cc:847
+    {
+      (yylhs.value.str) = (yystack_[0].value.str);
+    }
+#line 906 "neverclaimparse.cc" // lalr1.cc:847
+    break;
+
+  case 23:
+#line 216 "neverclaimparse.yy" // lalr1.cc:847
     {
       delete (yystack_[0].value.str);
       (yylhs.value.str) = new std::string("accept_all");
       accept_all_needed = true;
     }
-#line 890 "neverclaimparse.cc" // lalr1.cc:847
+#line 916 "neverclaimparse.cc" // lalr1.cc:847
     break;
 
-  case 22:
-#line 213 "neverclaimparse.yy" // lalr1.cc:847
+  case 24:
+#line 223 "neverclaimparse.yy" // lalr1.cc:847
     {
       // If there is no destination, do ignore the transition.
       // This happens for instance with
@@ -923,27 +949,27 @@ namespace neverclaimyy {
 	  (yylhs.value.p) = new pair(f, (yystack_[0].value.str));
 	}
     }
-#line 927 "neverclaimparse.cc" // lalr1.cc:847
+#line 953 "neverclaimparse.cc" // lalr1.cc:847
     break;
 
-  case 23:
-#line 249 "neverclaimparse.yy" // lalr1.cc:847
+  case 25:
+#line 259 "neverclaimparse.yy" // lalr1.cc:847
     {
       (yylhs.value.p) = (yystack_[1].value.p);
     }
-#line 935 "neverclaimparse.cc" // lalr1.cc:847
+#line 961 "neverclaimparse.cc" // lalr1.cc:847
     break;
 
-  case 24:
-#line 253 "neverclaimparse.yy" // lalr1.cc:847
+  case 26:
+#line 263 "neverclaimparse.yy" // lalr1.cc:847
     {
       (yylhs.value.p) = (yystack_[0].value.p);
     }
-#line 943 "neverclaimparse.cc" // lalr1.cc:847
+#line 969 "neverclaimparse.cc" // lalr1.cc:847
     break;
 
 
-#line 947 "neverclaimparse.cc" // lalr1.cc:847
+#line 973 "neverclaimparse.cc" // lalr1.cc:847
             default:
               break;
             }
@@ -1205,79 +1231,79 @@ namespace neverclaimyy {
   const signed char
   parser::yypact_[] =
   {
-       6,     3,    11,     8,   -13,     2,   -12,    -3,   -13,   -13,
-     -13,     8,   -13,   -13,   -13,   -13,     5,   -13,   -13,    -6,
-      -5,   -13,   -13,     9,   -13,   -13,     4,   -13,    10,   -13,
-      13,   -13,    -4,     7,   -13,    12,    15,    17,   -13,   -13,
-     -13
+       6,     5,    22,     8,   -13,     9,   -12,   -13,    -3,   -13,
+     -13,   -13,     8,   -13,   -13,   -13,   -13,   -13,   -13,   -13,
+      -6,    -5,   -13,    10,   -13,   -13,     4,   -13,    11,   -13,
+     -13,    15,   -13,    -4,     7,   -13,    13,    16,    12,   -13,
+     -13,   -13
   };
 
   const unsigned char
   parser::yydefact_[] =
   {
-       0,     0,     0,     3,     1,     0,     0,    12,     4,     7,
-       2,     6,    11,    15,    15,    13,     0,    14,     5,     0,
-       0,     8,     9,     0,    16,    10,     0,    18,     0,    17,
-      19,    24,     0,     0,    22,     0,     0,     0,    23,    20,
-      21
+       0,     0,     0,     3,     1,     0,     0,     8,    13,     4,
+       7,     2,     6,    12,    16,    16,    14,     9,    15,     5,
+       0,     0,    10,     0,    17,    11,     0,    20,     0,    18,
+      19,    21,    26,     0,     0,    24,     0,     0,     0,    25,
+      22,    23
   };
 
   const signed char
   parser::yypgoto_[] =
   {
-     -13,   -13,   -13,   -13,   -13,    14,    18,   -13,   -13,     1,
-     -13
+     -13,   -13,   -13,    17,   -13,   -13,    20,    18,   -13,   -13,
+       1,   -13
   };
 
   const signed char
   parser::yydefgoto_[] =
   {
-      -1,     2,     6,     7,    17,     8,    19,    30,    34,    31,
-      24
+      -1,     2,     6,     7,     8,    18,     9,    20,    31,    35,
+      32,    24
   };
 
   const unsigned char
   parser::yytable_[] =
   {
-      22,    12,    13,    25,    14,    10,    11,    27,    15,     1,
-      29,     4,    16,    23,    23,    27,    28,    36,    29,     3,
-      37,     9,    33,     5,    21,    18,    32,     0,    26,    38,
-      39,    40,    20,    35
+      22,    13,    14,    25,    15,    11,    12,    27,    16,     1,
+      29,    30,     5,    23,    23,    27,    28,    37,    29,    30,
+      38,     3,     4,     5,    34,    17,    41,    33,    10,    26,
+      39,    40,    19,    21,    36
   };
 
-  const signed char
+  const unsigned char
   parser::yycheck_[] =
   {
        6,     4,     5,     8,     7,    17,    18,    11,    11,     3,
-      14,     0,    15,    19,    19,    11,    12,    10,    14,    16,
-      13,    19,     9,    15,    19,    11,    16,    -1,    19,    17,
-      15,    14,    14,    32
+      14,    15,    15,    19,    19,    11,    12,    10,    14,    15,
+      13,    16,     0,    15,     9,     8,    14,    16,    19,    19,
+      17,    15,    12,    15,    33
   };
 
   const unsigned char
   parser::yystos_[] =
   {
-       0,     3,    21,    16,     0,    15,    22,    23,    25,    19,
-      17,    18,     4,     5,     7,    11,    15,    24,    25,    26,
-      26,    19,     6,    19,    30,     8,    19,    11,    12,    14,
-      27,    29,    16,     9,    28,    29,    10,    13,    17,    15,
-      14
+       0,     3,    21,    16,     0,    15,    22,    23,    24,    26,
+      19,    17,    18,     4,     5,     7,    11,    23,    25,    26,
+      27,    27,     6,    19,    31,     8,    19,    11,    12,    14,
+      15,    28,    30,    16,     9,    29,    30,    10,    13,    17,
+      15,    14
   };
 
   const unsigned char
   parser::yyr1_[] =
   {
-       0,    20,    21,    22,    22,    22,    22,    23,    23,    24,
-      24,    25,    25,    25,    25,    26,    26,    27,    27,    28,
-      28,    28,    29,    30,    30
+       0,    20,    21,    22,    22,    22,    22,    23,    24,    24,
+      25,    25,    26,    26,    26,    26,    27,    27,    28,    28,
+      28,    29,    29,    29,    30,    31,    31
   };
 
   const unsigned char
   parser::yyr2_[] =
   {
-       0,     2,     4,     0,     1,     3,     2,     2,     3,     3,
-       3,     2,     1,     2,     2,     0,     2,     1,     1,     0,
-       3,     3,     2,     6,     3
+       0,     2,     4,     0,     1,     3,     2,     2,     1,     2,
+       3,     3,     2,     1,     2,     2,     0,     2,     1,     1,
+       1,     0,     3,     3,     2,     6,     3
   };
 
 
@@ -1291,17 +1317,17 @@ namespace neverclaimyy {
   "\"fi\"", "\"do\"", "\"od\"", "\"->\"", "\"goto\"", "\"false\"",
   "\"atomic\"", "\"assert\"", "\"boolean formula\"", "\"identifier\"",
   "'{'", "'}'", "';'", "':'", "$accept", "neverclaim", "states",
-  "ident_list", "transition_block", "state", "transitions", "formula",
-  "opt_dest", "src_dest", "transition", YY_NULLPTR
+  "one_ident", "ident_list", "transition_block", "state", "transitions",
+  "formula", "opt_dest", "src_dest", "transition", YY_NULLPTR
   };
 
 #if YYDEBUG
-  const unsigned char
+  const unsigned short int
   parser::yyrline_[] =
   {
-       0,   102,   102,   105,   107,   108,   109,   112,   116,   134,
-     138,   144,   156,   157,   158,   182,   183,   194,   194,   198,
-     201,   205,   212,   248,   252
+       0,   103,   103,   106,   108,   109,   110,   112,   125,   126,
+     144,   148,   154,   166,   167,   168,   192,   193,   204,   204,
+     204,   208,   211,   215,   222,   258,   262
   };
 
   // Print the state stack on the debug stream.
@@ -1384,8 +1410,8 @@ namespace neverclaimyy {
 
 
 } // neverclaimyy
-#line 1388 "neverclaimparse.cc" // lalr1.cc:1155
-#line 256 "neverclaimparse.yy" // lalr1.cc:1156
+#line 1414 "neverclaimparse.cc" // lalr1.cc:1155
+#line 266 "neverclaimparse.yy" // lalr1.cc:1156
 
 
 void
@@ -1427,6 +1453,7 @@ namespace spot
       }
     accept_all_needed = false;
     accept_all_seen = false;
+    labels.clear();
 
     return result;
   }
