@@ -33,6 +33,7 @@
 #include "ltlvisit/tostring.hh"
 #include <cassert>
 #include <memory>
+#include <algorithm>
 #include "ltl2tgba_fm.hh"
 #include "tgba/bddprint.hh"
 #include "tgbaalgos/scc.hh"
@@ -254,24 +255,59 @@ namespace spot
 	// Similarly
 	//      a M b = (a R (b&P(a)))
 	//      (a M b) M c = (a R (b & Pa)) R (c & P(a M b))
-	//                  = (a R (b & Pa)) R (c & P(a))
-	// The rules also apply to F:
-	//   P(F(a)) = P(a)
-      again:
-	while (const binop* b = is_binop(f))
+	//                  = (a R (b & Pa)) R (c & P(a & b))
+	//
+	// The code below therefore implement the following
+	// rules:
+	// P(a U b) = P(b)
+	// P(F(a))  = P(a)
+	// P(a M b) = P(a & b)
+	//
+	// The latter rule INCORRECTLY appears as P(a M b)=P(a)
+	// in section 3.5 of
+	//    "LTL translation improvements in Spot 1.0",
+	//     A. Duret-Lutz. IJCCBS 5(1/2):31-54, March 2014.
+	// and was unfortunately implemented this way until Spot
+	// 1.2.4.  A counterexample is given by the formula
+	//    G(Fa & ((a M b) U ((c U !d) M d)))
+	// that was found by Joachim Klein.  Here P((c U !d) M d)
+	// and P(c U !d) should not both be simplified to P(!d).
+	for (;;)
 	  {
-	    binop::type op = b->op();
-	    if (op == binop::U)
-	      f = b->second();
-	    else if (op == binop::M)
-	      f = b->first();
+	    if (const binop* b = is_binop(f))
+	      {
+		binop::type op = b->op();
+		if (op == binop::U)
+		  {
+		    // P(a U b) = P(b)
+		    f = b->second();
+		  }
+		else if (op == binop::M)
+		  {
+		    // P(a M b) = P(a & b)
+		    const formula* g =
+		      multop::instance(multop::And,
+				       b->first()->clone(),
+				       b->second()->clone());
+		    int num = dict->register_acceptance_variable(g, this);
+		    a_set &= bdd_ithvar(num);
+		    g->destroy();
+		    return num;
+		  }
+		else
+		  {
+		    break;
+		  }
+	      }
+	    else if (const unop* u = is_unop(f, unop::F))
+	      {
+		// P(F(a)) = P(a)
+		f = u->child();
+	      }
 	    else
-	      break;
-	  }
-	if (const unop* u = is_unop(f, unop::F))
-	  {
-	    f = u->child();
-	    goto again;
+	      {
+		break;
+	      }
 	  }
 	int num = dict->register_acceptance_variable(f, this);
 	a_set &= bdd_ithvar(num);
@@ -1403,48 +1439,51 @@ namespace spot
 	  case unop::NegClosure:
 	    rat_seen_ = true;
 	    {
-	      const formula* c = node->child();
 	      if (mark_all_)
 		{
 		  op = unop::NegClosureMarked;
 		  has_marked_ = true;
 		}
-	      bdd f1 = translate_ratexp(c, dict_);
 
-	      // trace_ltl_bdd(dict_, f1);
+	      const formula* f = node->child();
+	      tgba_succ_iterator* i = dict_.transdfa.succ(f);
+	      res_ = bddtrue;
 
-	      bdd var_set = bdd_existcomp(bdd_support(f1), dict_.var_set);
-	      bdd all_props = bdd_existcomp(f1, dict_.var_set);
+	      if (!i)
+		break;
 
-	      res_ = !all_props &
-		// stick X(1) to preserve determinism.
-		bdd_ithvar(dict_.register_next_variable
-			   (constant::true_instance()));
-
-	      while (all_props != bddfalse)
+	      res_ = bddfalse;
+	      bdd missing = bddtrue;
+	      for (i->first(); !i->done(); i->next())
 		{
-		  bdd label = bdd_satoneset(all_props, var_set, bddtrue);
-		  all_props -= label;
+		  bdd label = i->current_condition();
+		  state* s = i->current_state();
+		  const formula* dest = dict_.transdfa.get_label(f, s);
+		  s->destroy();
 
-		  const formula* dest =
-		    dict_.bdd_to_sere(bdd_exist(f1 & label, dict_.var_set));
+		  missing -= label;
 
-		  // !{ Exp } is false if Exp accepts the empty word.
 		  if (dest->accepts_eword())
 		    {
 		      dest->destroy();
-		      continue;
 		    }
-
-		  const formula* dest2 = unop::instance(op, dest);
-
-		  if (dest2 == constant::false_instance())
-		    continue;
-
-		  int x = dict_.register_next_variable(dest2);
-		  dest2->destroy();
-		  res_ |= label & bdd_ithvar(x);
+		  else
+		    {
+		      const formula* dest2 = unop::instance(op, dest);
+		      if (dest2 == constant::false_instance())
+			continue;
+		      int x = dict_.register_next_variable(dest2);
+		      dest2->destroy();
+		      res_ |= label & bdd_ithvar(x);
+		    }
 		}
+	      delete i;
+
+	      res_ |= missing &
+		// stick X(1) to preserve determinism.
+		bdd_ithvar(dict_.register_next_variable
+			   (constant::true_instance()));
+	      //trace_ltl_bdd(dict_, res_);
 	    }
 	    break;
 
@@ -1521,9 +1560,23 @@ namespace spot
 	    {
 	      res_ = recurse(node->second(), recurring_);
 	      bdd f1 = recurse(node->first());
-	      // r(f1 M f2) = r(f2)(r(f1) + a(f1)X(f1 M f2)) if not recurring
-	      // r(f1 M f2) = r(f2)(r(f1) + a(f1))           if recurring
-	      bdd a = bdd_ithvar(dict_.register_a_variable(node->first()));
+	      // r(f1 M f2) = r(f2)(r(f1) + a(f1&f2)X(f1 M f2)) if not recurring
+	      // r(f1 M f2) = r(f2)(r(f1) + a(f1&f2))           if recurring
+	      //
+	      // Note that the rule above differs from the one given
+	      // in Figure 2 of
+	      //    "LTL translation improvements in Spot 1.0",
+	      //     A. Duret-Lutz. IJCCBS 5(1/2):31-54, March 2014.
+	      // Both rules should be OK, but this one is a better fit
+	      // to the promises simplifications performed in
+	      // register_a_variable() (see comments in this function).
+	      // We do not want a U (c M d) to generate two different
+	      // promises.  Generating c&d also makes the output similar
+	      // to what we would get with the equivalent a U (d U (c & d)).
+	      //
+	      // Here we just appear to emit a(f1 M f2) and the conversion
+	      // to a(f1&f2) is done  by register_a_variable().
+	      bdd a = bdd_ithvar(dict_.register_a_variable(node));
 	      if (!recurring_)
 		a &= bdd_ithvar(dict_.register_next_variable(node));
 	      res_ &= f1 | a;
@@ -2057,28 +2110,56 @@ namespace spot
 
   }
 
-  typedef std::map<bdd, bdd, bdd_less_than> prom_map;
-  typedef Sgi::hash_map<const formula*, prom_map, formula_ptr_hash> dest_map;
-
-  static void
-  fill_dests(translate_dict& d, dest_map& dests, bdd label, const formula* dest)
+  namespace
   {
-    bdd conds = bdd_existcomp(label, d.var_set);
-    bdd promises = bdd_existcomp(label, d.a_set);
+    struct transition
+    {
+      const formula* dest;
+      bdd prom;
+      bdd cond;
 
-    dest_map::iterator i = dests.find(dest);
-    if (i == dests.end())
+      transition(const formula* dest, bdd cond, bdd prom)
+	: dest(dest), prom(prom), cond(cond)
       {
-	dests[dest][promises] = conds;
       }
-    else
+
+      transition(const transition& other)
+	: dest(other.dest), prom(other.prom), cond(other.cond)
       {
-	i->second[promises] |= conds;
-	dest->destroy();
       }
+
+      bool operator<(const transition& other) const
+      {
+	ltl::formula_ptr_less_than lt;
+	if (lt(dest, other.dest))
+	  return true;
+	if (lt(other.dest, dest))
+	  return false;
+	if (prom.id() < other.prom.id())
+	  return true;
+	if (prom.id() > other.prom.id())
+	  return false;
+	return cond.id() < other.cond.id();
+      }
+    };
+
+    bool postponement_cmp(const transition& lhs, const transition& rhs)
+    {
+      if (lhs.prom.id() < rhs.prom.id())
+	return true;
+      if (lhs.prom.id() > rhs.prom.id())
+	return false;
+      if (lhs.cond.id() < rhs.cond.id())
+	return true;
+      if (lhs.cond.id() > rhs.cond.id())
+	return false;
+      ltl::formula_ptr_less_than lt;
+      return lt(lhs.dest, rhs.dest);
+    }
+
+    typedef std::vector<transition> dest_map;
   }
 
-
   tgba_explicit_formula*
   ltl_to_tgba_fm(const formula* f, bdd_dict* dict,
 		 bool exprop, bool symb_merge, bool branching_postponement,
@@ -2168,6 +2249,7 @@ namespace spot
     formulae_to_translate.insert(f2);
     a->set_init_state(f2);
 
+    dest_map dests;
     while (!formulae_to_translate.empty())
       {
 	// Pick one formula.
@@ -2178,6 +2260,9 @@ namespace spot
 	const translate_dict::translated& t = fc.translate(now);
 	bdd res = t.symbolic;
 
+	if (res == bddfalse)
+	  continue;
+
 	// Handle exclusive events.
 	if (unobs)
 	  {
@@ -2215,7 +2300,7 @@ namespace spot
 	//
 	// In `exprop' mode, considering all possible combinations of
 	// outgoing propositions generalizes the above trick.
-	dest_map dests;
+	dests.clear();
 
 	// Compute all outgoing arcs.
 
@@ -2236,9 +2321,6 @@ namespace spot
 	      one_prop_set = bdd_satoneset(all_props, var_set, bddtrue);
 	    all_props -= one_prop_set;
 
-	    typedef std::map<bdd, const formula*, bdd_less_than> succ_map;
-	    succ_map succs;
-
 	    // Compute prime implicants.
 	    // The reason we use prime implicants and not bdd_satone()
 	    // is that we do not want to get any negation in front of Next
@@ -2277,38 +2359,36 @@ namespace spot
 		if (symb_merge)
 		  dest = fc.canonize(dest);
 
-		// If we are not postponing the branching, we can
-		// declare the outgoing transitions immediately.
-		// Otherwise, we merge transitions with identical
-		// label, and declare the outgoing transitions in a
-		// second loop.
-		if (!branching_postponement)
-		  {
-		    fill_dests(d, dests, label, dest);
-		  }
-		else
-		  {
-		    succ_map::iterator si = succs.find(label);
-		    if (si == succs.end())
-		      succs[label] = dest;
-		    else
-		      si->second =
-			multop::instance(multop::Or, si->second, dest);
-		  }
+		bdd conds = bdd_existcomp(label, d.var_set);
+		bdd promises = bdd_existcomp(label, d.a_set);
+		dests.push_back(transition(dest, conds, promises));
 	      }
-	    if (branching_postponement)
-	      for (succ_map::const_iterator si = succs.begin();
-		   si != succs.end(); ++si)
-		fill_dests(d, dests, si->first, si->second);
 	  }
 
-	// Check for an arc going to 1 (True).  Register it first, that
-	// way it will be explored before others during model checking.
-	dest_map::const_iterator i = dests.find(constant::true_instance());
-	// COND_FOR_TRUE is the conditions of the True arc, so we
-	// can remove them from all other arcs.  It might sounds that
-	// this is not needed when exprop is used, but in fact it is
-	// complementary.
+	assert(dests.size() > 0);
+	if (branching_postponement && dests.size() > 1)
+	  {
+	    std::sort(dests.begin(), dests.end(), postponement_cmp);
+	    // Iterate over all dests, and merge the destination of
+	    // transitions with identical labels.
+	    dest_map::iterator out = dests.begin();
+	    dest_map::const_iterator in = out;
+	    do
+	      {
+		transition t = *in;
+		while (++in != dests.end()
+		       && t.cond == in->cond && t.prom == in->prom)
+		  t.dest = multop::instance(multop::Or, t.dest, in->dest);
+		*out++ = t;
+	      }
+	    while (in != dests.end());
+	    dests.erase(out, dests.end());
+	  }
+	std::sort(dests.begin(), dests.end());
+	// If we have some transitions to true, they are the first
+	// ones.  Remove the sum of their conditions from other
+	// transitions.  It might sounds that this is not needed when
+	// exprop is used, but in fact it is complementary.
 	//
 	// Consider
 	//   f = r(X(1) R p) = p.(1 + r(X(1) R p))
@@ -2321,67 +2401,59 @@ namespace spot
 	//     f ----> 1
 	//
 	// because there is no point in looping on f if we can go to 1.
-	bdd cond_for_true = bddfalse;
-	if (i != dests.end())
+	if (dests.front().dest == constant::true_instance())
 	  {
-	    // When translating LTL for an event-based logic with
-	    // unobservable events, the 1 state should accept all events,
-	    // even unobservable events.
-	    if (unobs && now == constant::true_instance())
-	      cond_for_true = all_events;
-	    else
-	      {
-		// There should be only one transition going to 1 (true) ...
-		assert(i->second.size() == 1);
-		prom_map::const_iterator j = i->second.begin();
-		// ... and it is not expected to make any promises (unless
-		// fair loop approximations are used).
-		assert(fair_loop_approx || j->first == bddtrue);
-		cond_for_true = j->second;
-	      }
-	    if (!a->has_state(constant::true_instance()))
-	      formulae_to_translate.insert(constant::true_instance());
-	    state_explicit_formula::transition* t =
-	      a->create_transition(now, constant::true_instance());
-	    t->condition = cond_for_true;
+	    dest_map::iterator i = dests.begin();
+	    bdd c = bddfalse;
+	    while (i != dests.end() && i->dest == constant::true_instance())
+	      c |= i++->cond;
+	    for (; i != dests.end(); ++i)
+	      i->cond -= c;
 	  }
-	// Register other transitions.
-	for (i = dests.begin(); i != dests.end(); ++i)
-	  {
-	    const formula* dest = i->first;
-	    // The cond_for_true optimization can cause some
-	    // transitions to be removed.  So we have to remember
-	    // whether a formula is actually reachable.
-	    bool reachable = false;
 
-	    // Will this be a new state?
-	    bool seen = a->has_state(dest);
+	// Create transitions in the automaton
+	{
+	  dest_map::const_iterator in = dests.begin();
+	  do
+	    {
+	      // Merge transitions with same destination and
+	      // acceptance.
+	      transition t = *in;
+	      while (++in != dests.end()
+		     && t.prom == in->prom && t.dest == in->dest)
+		{
+		  t.cond |= in->cond;
+		  in->dest->destroy();
+		}
+	      // Actually create the transition
+	      if (t.cond != bddfalse)
+		{
+		  // When translating LTL for an event-based logic
+		  // with unobservable events, the 1 state should
+		  // accept all events, even unobservable events.
+		  if (unobs
+		      && t.dest == constant::true_instance()
+		      && now == constant::true_instance())
+		    t.cond = all_events;
 
-	    if (dest != constant::true_instance())
-	      {
-		for (prom_map::const_iterator j = i->second.begin();
-		     j != i->second.end(); ++j)
-		  {
-		    bdd cond = j->second - cond_for_true;
-		    if (cond == bddfalse) // Skip false transitions.
-		      continue;
-		    state_explicit_formula::transition* t =
-		      a->create_transition(now, dest);
-		    t->condition = cond;
-		    d.conj_bdd_to_acc(a, j->first, t);
-		    reachable = true;
-		  }
-	      }
-	    else
-	      {
-		// "1" is reachable.
-		reachable = true;
-	      }
-	    if (reachable && !seen)
-	      formulae_to_translate.insert(dest);
-	    else
-	      dest->destroy();
-	  }
+		  // Will this be a new state?
+		  bool seen = a->has_state(t.dest);
+
+		  state_explicit_formula::transition* tt =
+		    a->create_transition(now, t.dest);
+		  tt->condition = t.cond;
+		  d.conj_bdd_to_acc(a, t.prom, tt);
+
+		  if (!seen)
+		    formulae_to_translate.insert(t.dest);
+		  else
+		    t.dest->destroy();
+		}
+	      else
+		t.dest->destroy();
+	    }
+	  while (in != dests.end());
+	}
       }
 
     dict->register_propositions(fc.used_vars(), a);
